﻿-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...

local max = math.max

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local ldbicon = ldb and LibStub("LibDBIcon-1.0", true)
addonTable.ldbicon = ldbicon

local L = addonTable.L -- localization
local C = addonTable.C -- constants
local T = addonTable.T -- textures
local ng = NyxGUI(C.NyxGUIVersion)

-- string colors
local AddColor = addonTable.AddColor

local addontitle = "|cff00AAFFDaily Global Check|r"
local addontitle_short = "|cff00AAFFDGC|r"

-- constants
local ZONE = 1
local NAME = 2
local PREF = 3
local SUFF = 4
local MAPDATA = 5
local MAPDEFAULT = 6
local QUESTTYPE = 7 -- nil == daily
local MAPICON = 8 -- nil == plugin icon
local SHOWFUNC = 9
local COMPLETEDFUNC = 10
local PLUGIN = 11
local HIDDEN = 12 -- hidden in the editframe quests pool
local OVERLAY = 13

function DailyGlobalCheck:GetVersion()
 return C.version
end

-- saved variables
DailyGlobalCheck_Options = {}
DGC_CustomLists = {}
DailyGlobalCheck_CharData = {}
DailyGlobalCheck_PluginData = {}

-- ext
local chars        = DailyGlobalCheck.CharFunctions
local pages        = DailyGlobalCheck.PagesFunctions
local options      = DailyGlobalCheck.OptFunctions
local constructors = DailyGlobalCheck.Constructors
local lists        = DailyGlobalCheck.ListFunctions
local maps         = DailyGlobalCheck.MapFunctions
local quest        = DailyGlobalCheck.QuestFunctions
local modes        = DailyGlobalCheck.ModeFunctions
local tools        = DailyGlobalCheck.Tools

local charData = DailyGlobalCheck_CharData
local pName = GetUnitName("player")
local pRealm = GetRealmName()
addonTable.pID = pName.."-"..pRealm
local pID = addonTable.pID
-- init
DailyGlobalCheck.LoadQueue = {}
-- pools
DailyGlobalCheck.Lists = {}
--
DailyGlobalCheck.player_faction = UnitFactionGroup("player")
DailyGlobalCheck.selectedpID = pID
---
local SelectedList
local DGCTTmenu = {}
local DGCMMmenu = {}
--
-- quests data pool
DailyGlobalCheck.QuestsData = {}
local questsdata = DailyGlobalCheck.QuestsData
local questsdata_duplicates = {}
local counter = 0

---- metatables ----
local function setmeta(t, meta)
 if not getmetatable(t) then
  setmetatable(t, meta)
 end
end

local questdata_template = {"", L["namemissing"], "", "", nil, nil, "D"}
local meta_qd_template = { __index = questdata_template }
local function meta_qd_template_newindex(t, k, v)
 rawset(t, k, v)
 if v then
  setmeta(v, meta_qd_template)
 end
end
local meta_qd_newindex = { __newindex = meta_qd_template_newindex }
--------

function DailyGlobalCheck:GetSelectedList()
 return SelectedList
end

-- fixed lists (plugins) and custom lists (saved variables)
function DailyGlobalCheck:foreachLists(func, active)
 for k,v in pairs(DailyGlobalCheck.Lists) do
  if active == false or lists:isActive(v) then
   func(k,v)
  end
 end
 
 if not DGC_CustomLists.Lists then return end
 
 for k,v in pairs(DGC_CustomLists.Lists) do
  if active == false or lists:isActive(v) then
   func(k,v)
  end
 end
end

function DailyGlobalCheck:foreachQData(func)
 for k,v in pairs(DailyGlobalCheck.QuestsData) do
  func(k,v)
 end

 for k,v in pairs(questsdata_duplicates) do
  for plugin,data in pairs(v) do
   func(plugin.."<!"..k, data)
  end
 end

 if not DGC_CustomLists.QuestsData then return end
 
 for k,v in pairs(DGC_CustomLists.QuestsData) do
  func(k,v)
 end
end

function DailyGlobalCheck:getQData(questID, plugin)
 if plugin then
  local s = plugin.."<!"..questID
  return DailyGlobalCheck.QuestsData[s] or DailyGlobalCheck.QuestsData[questID]
 else
  return DailyGlobalCheck.QuestsData[questID] or DGC_CustomLists.QuestsData[questID]
 end
end
--

function DailyGlobalCheck:NormalizeQuestID(questID)
 local plugin, qID
 local i = strfind(questID, "<!")
 
 if i then
  plugin  = questID:sub(1, i - 1)
  qID = questID:sub(i + 2, #questID)
  return qID, plugin
 else
  return questID
 end 
end

------------------------------------------------------------------------------
function DailyGlobalCheck:GetCoord(data)

 local function beautify(coord)
  local s = tostring(math.floor(coord * 10) / 10)
  local i = strfind(s, "%.")
  if i then
   if i < 2 then s = "0"..s end
  else
   s = s..".0"
  end
  if coord < 10 then s = "0"..s end
  return s
 end

 local v = data[MAPDATA]
 local default = data[MAPDEFAULT]
 return (v and default) and AddColor(beautify(v[default][1]).."  "..beautify(v[default][2]), DailyGlobalCheck_Options.Colors[5]) or ""
end
------------------------------------------------------------------------------
function DailyGlobalCheck.isquestcompleted(questID, mainCharOnly, notSelectedPlugin)

 local list = notSelectedPlugin or SelectedList

 if not mainCharOnly and DailyGlobalCheck.selectedpID ~= pID and list.MultiCharsEnabled then
  return chars:isquestcompleted(questID, DailyGlobalCheck.selectedpID)
 end

 local data = DailyGlobalCheck:getQData(questID, list and list["Title"] or nil)

 questID = DailyGlobalCheck:NormalizeQuestID(questID)

 if data and data[COMPLETEDFUNC] then
  return data[COMPLETEDFUNC](questID)
 elseif type(questID) ~= "number" then
  local result, cs, reset = quest:isquestcompleted(questID)
  return result, cs, reset
 else
  return IsQuestFlaggedCompleted(questID)
 end
end
local isquestcompleted = DailyGlobalCheck.isquestcompleted
------------------------------------------------------------------------------
local function isshown(questID)
 local qData = DailyGlobalCheck:getQData(questID)
 if not qData then return false end

  -- fire the show func if present
 return not qData[SHOWFUNC] or qData[SHOWFUNC](questID)
end
addonTable.isshown = isshown

function DailyGlobalCheck:RefreshTitle()
 local f = DGC_Mainframe
 
 if not f then return end

 local plugintitle = SelectedList and AddColor(SelectedList["Title"], "LIST_COLOR") or ""
 local mode = DailyGlobalCheck.mode or 1
 
 if mode ~= C.MODE_CLASSIC then
  if mode == C.MODE_OPTIONS_PLUGIN then
   f.title:SetText(SelectedList and SelectedList["Title"] or "")
   f.page:SetText("")
  elseif mode == C.MODE_EDIT then
   f.title:SetText(SelectedList and SelectedList["Title"] or "")
   f.page:SetText(pages:GetString(SelectedList))
  else
   f.title:SetText("")
   f.page:SetText("")
  end
  
  f.Ptitle:SetText(AddColor(C.modes[mode].name, "LIST_COLOR"))
  return
 end

 if SelectedList then
  local suff = pID ~= DailyGlobalCheck.selectedpID and AddColor(" "..L["opt_characters_short"], "LIGHT_BLUE") or ""
  --local title = SelectedList.MultiCharsEnabled and DailyGlobalCheck.selectedpID..suff or ""
  local name = strsplit("-", (DailyGlobalCheck.selectedpID or ""))
  local title = SelectedList.MultiCharsEnabled and AddColor(name..suff, chars:GetCharClass(DailyGlobalCheck.selectedpID)) or ""
  f.title:SetText(title)
  -- pages
  f.page:SetText(pages:GetString(SelectedList))
 end
 f.Ptitle:SetText(plugintitle)
end
---------------------------- TomTom Menu Frame -------------------------------------
local function setTomTomWaypoint(...)
 TomTom:AddMFWaypoint(...)
 local t = select(5, ...)
 print(L["s_tomtomset"].." ("..AddColor(t.title, "LIGHT_BLUE")..")")
end
addonTable.setTomTomWP = setTomTomWaypoint -- temporary until I fix the TomTom interface

function DailyGlobalCheck:ShowTomTomMenu()
 if not SelectedList then return end
 DGCMenuFrame:SetPoint("TOP",DGC_Mainframe.TomTombutton,"BOTTOM",0,-5)
 EasyMenu(DGCTTmenu, DGCMenuFrame, DGCMenuFrame, 0 , 0, "MENU")
end

local function GenerateTomTomMenu()

 if not TomTom or not DGCMenuFrame or not SelectedList then return end

  local function CreateSubMenu(q)
   local submenu = {}
   for index,questID in pairs(q) do
    local data = DailyGlobalCheck:getQData(questID)
    if index > 1 and data and isshown(questID) and data[MAPDATA] then
    table.insert(submenu, { text = data[NAME].." - "..AddColor(data[ZONE],"GRAY").." "..DailyGlobalCheck:GetCoord(data),
                            disabled = isquestcompleted(questID) or not data[MAPDATA],
                            notCheckable = 1,
                            keepShownOnClick = true,
                            func = function(self)
                               if data and data[MAPDATA] then
                                 setTomTomWaypoint(data[MAPDEFAULT], nil,
                                            data[MAPDATA][data[MAPDEFAULT]][1] / 100,
                                            data[MAPDATA][data[MAPDEFAULT]][2] / 100,
                                          { title = data[NAME]})
                               end
                              end
                              })
     end
    end
    return submenu
   end

  wipe(DGCTTmenu)
  table.insert(DGCTTmenu, {text = AddColor("- TomTom Waypoints -","GOLD"),notCheckable = 1, isTitle = true})
  table.insert(DGCTTmenu, {text = AddColor("Click either a single quest or a group","LIGHT_BLUE"),notCheckable = 1, isTitle = true})
  table.insert(DGCTTmenu, {text = AddColor("to set TomTom waypoints","LIGHT_BLUE"),notCheckable = 1, isTitle = true})
  table.insert(DGCTTmenu, {text = "",notCheckable = 1, isTitle = true})

  table.foreach(SelectedList["Order"], function(k,tab)
   local pref = "|T"..SelectedList["Icon"]..":12|t"
   table.foreach(tab, function(header,quests)
    if table.getn(quests) > 1 then
    table.insert(DGCTTmenu, { text = quests[1] ~= "" and pref..quests[1] or pref..L["notitle"],
                              keepShownOnClick = true,
                              notCheckable = 1,
                       func = function(self)
                               table.foreach(quests, function(index, questID)
							    if index > 1 then
 								 local data = DailyGlobalCheck:getQData(questID)
                                  if data and data[MAPDATA] and not isquestcompleted(questID) and isshown(questID) then
                                   TomTom:AddMFWaypoint(data[MAPDEFAULT], nil,
                                              data[MAPDATA][data[MAPDEFAULT]][1] / 100,
                                              data[MAPDATA][data[MAPDEFAULT]][2] / 100,
                                            { title = data[NAME] } )
                                   print(L["s_tomtomset"].." ("..AddColor(data[NAME],"LIGHT_BLUE")..")")
                                 end
								end
                               end)
                              end,
                       hasArrow = true,
		               menuList = CreateSubMenu(quests)})
	end
   end)
  end)
  table.insert(DGCTTmenu, {text = "", isTitle = true, notCheckable = 1})
  table.insert(DGCTTmenu, {text = CLOSE, notCheckable = 1})
end

---------------------------- Generate Line (used by drawtooltip and /dgc print) -------------------------------------
local function GenerateLine(questID, cstring)
 local s1 = ""
 local O = DailyGlobalCheck_Options
 local data = DailyGlobalCheck:getQData(questID)
 local questtype = data[QUESTTYPE]

 if not data then return L["questdatamissing"] end

 -- quest type
 if O["show_questtype"] then
  s1 = s1..AddColor("["..questtype.."]", "LIGHT_BLUE") 
 end
 -- prefix
 if O["show_prefix"] then
  s1 = s1..AddColor(data[PREF], O.Colors[3])
 end
 -- zone name
 if O["show_zone"] then
  s1 = s1..AddColor(data[ZONE], O.Colors[1])
 end
 -- quest name
 s1 = s1.." "..AddColor(data[NAME], O.Colors[2])
 -- suffix
 if O["show_suffix"] then
  s1 = s1..AddColor(data[SUFF].." ", O.Colors[4])
 end
 -- coordinates
 if O["show_coordinates"] then
  s1 = s1.." "..AddColor(DailyGlobalCheck:GetCoord(data), O.Colors[5])
 end
 
 return s1, questtype
end
------------------------------------------ Draw Tooltip --------------------------------------------------------
local function DrawTooltip(tt)
 local O = DailyGlobalCheck_Options

 if not SelectedList then
  if lists:Count() == 0 then
   tt:AddDoubleLine(AddColor("No active lists","LIGHT_RED")," ")
  else
   tt:AddDoubleLine(AddColor(L["nolist"],"LIGHT_RED")," ")
  end
 else
  local questsorder = SelectedList["Order"]
  local index = pages:Curr()
  
  if SelectedList["OnShow"] then SelectedList["OnShow"]() end
  options:OnDraw()
  
  if not questsorder[index] then return end
  
  local s1,s2,qt
  for k,v in pairs(questsorder[index]) do
   if v[1] ~= "" then
    tt:AddDoubleLine(AddColor(v[1], O.Colors[6])," ")
   end
   for i = 2,#v do
    local questID = v[i]
	 if isshown(questID) then
	  local completed, cstring, reset = isquestcompleted(questID, true)
	  if not DailyGlobalCheck_Options["hide_completed"] or (not completed or cstring or reset) then
	   if cstring == nil then 
	    cstring = completed and COMPLETE or INCOMPLETE
	   end
       s1, qt = GenerateLine(questID)
       s2 = qt == "R" and AddColor(L["repeatable"], O.Colors[9])                      -- repeatable
	                   or AddColor(cstring, completed and O.Colors[7] or O.Colors[8]) -- custom cstring or complete/incomplete
       tt:AddDoubleLine(s1,s2,questID)
	  end
	 end
    end
    tt:AddDoubleLine(" "," ")
   end
  end
  
 tt:AddLine(pages:GetString(SelectedList))
 tt:AddLine(AddColor(L["lclicktoopen"],"LIGHT_BLUE"))
 tt:AddLine(AddColor(L["rclicktoopen"],"LIGHT_BLUE"))
 tt:Show()
end

------------------------------------------ Draw Frame (fill button) ----------------------------------------------------
local columns_w = {}

local function getBtn(i)
 local result = DGC_Mainframe.panels[1].buttons[i] or constructors:QuestButton(i, DGC_Mainframe.panels[1])
 
 if not DGC_Mainframe.panels[1].buttons[i] then
  DGC_Mainframe.panels[1].buttons[i] = result
 end
 
 return result
end

local function adjustColumnsSize(f)
 columns_w[ZONE] = max(columns_w[ZONE] or 0, f.zone:GetStringWidth())
 columns_w[NAME] = max(columns_w[NAME] or 0, f.name:GetStringWidth())
 columns_w[PREF] = max(columns_w[PREF] or 0, f.prefix:GetStringWidth())
 columns_w[SUFF] = max(columns_w[SUFF] or 0, f.suffix:GetStringWidth())
 columns_w[5]    = max(columns_w[5]    or 0, f.coords:GetStringWidth())
 columns_w[6]    = max(columns_w[6]    or 0, f.completed:GetStringWidth())
end

local function fillButtonText(i, s1, s2)

 local f = getBtn(i)
 f:FillText(s1, s2)
 return f
end

local function fillButtonData(i, questID, cstring)

 local f = getBtn(i) 
 f:FillDataByID(questID, cstring, SelectedList["Title"]) 
 adjustColumnsSize(f)
 return f
end

------------------------------------------ Draw Frame --------------------------------------------------------
local function DrawFrame()
 
 local f = DGC_Mainframe
 
 local button = 1
 wipe(columns_w)
 
 ---
 local function fillButton(t, ...)
  local btn
  if t == "data" then
   btn = fillButtonData(button, ...)
   btn.noresize = nil
  elseif t == "text" then
   btn = fillButtonText(button, ...)
   btn.noresize = true
  else
   return
  end
  btn:Show()
  button = button + 1
 end
 ---
 
 if not SelectedList then
  if lists:Count() == 0 then
   fillButton("text", AddColor("No Lists Available", "LIGHT_RED"))
  else
   fillButton("text", AddColor(L["nolist"], "LIGHT_RED"))
  end
  -- hide the remaining buttons
  f.panels[1]:HideQuestButtons(2)
  f:SetSize(300, 150)
  return
 end
 
 local questsorder = SelectedList["Order"]
 local index = pages:Curr()

 if SelectedList["OnShow"] then SelectedList["OnShow"]() end
 options:OnDraw()

 if not questsorder[index] then return end

 for k,v in pairs(questsorder[index]) do
  if v[1] ~= "" then
   fillButton("text", AddColor(v[1], DailyGlobalCheck_Options.Colors[6]))
  end

  for i = 2,#v do
   local questID = v[i]
   if isshown(questID) then
    local completed, cstring, reset = isquestcompleted(questID, istooltip)
    if not DailyGlobalCheck_Options["hide_completed"] or (not completed or cstring or reset) then
     if cstring == nil then cstring = completed and COMPLETE or INCOMPLETE end
	 fillButton("data", questID, AddColor(cstring, completed and DailyGlobalCheck_Options.Colors[7] or DailyGlobalCheck_Options.Colors[8]))
    end
   end
  end
  fillButton("text", " ")
 end

 -- hide the remaining buttons
 f.panels[1]:HideQuestButtons(button - 1)

 -- create some space for the buttons
 local offset = C.pluginbtnsize * 5

 local w = 0
 
 for i = 1,6 do
  w = w + (columns_w[i] or 0)
 end
 
 f:SetSize(max(300, w + 30), button * 12 + offset)
 
 for i = 1, button - 1 do
  if not f.panels[1].buttons[i].noresize then
   f.panels[1].buttons[i].zone:SetWidth(columns_w[ZONE] or 0)
   f.panels[1].buttons[i].name:SetWidth(columns_w[NAME] or 0)
   f.panels[1].buttons[i].prefix:SetWidth(columns_w[PREF] or 0)
   f.panels[1].buttons[i].suffix:SetWidth(columns_w[SUFF] or 0)
   f.panels[1].buttons[i].coords:SetWidth(columns_w[5] or 0)
   f.panels[1].buttons[i].completed:SetWidth(columns_w[6] or 0)
  end
 end
 local title = DGC_Mainframe.Ptitle
 local w = title:GetWidth()
 title:SetPoint("TOPLEFT", DGC_Mainframe, "TOPLEFT", (DGC_Mainframe:GetWidth() - 70) / 2 - w / 2, -15)
end
---------------------------- Chat Output ---------------------------------------------
local function ChatOutput()
 if not SelectedList or not SelectedList["Order"][1] then return end

 local s1,s2,qt
 for k,v in pairs(SelectedList["Order"][1]) do
 if v[1] ~= "" then
  print(AddColor(v[1], DailyGlobalCheck_Options.Colors[6]))
 end
 for i,questID in pairs(v) do
        if i > 1 then
	    if isshown(questID) then
			local completed, cstring = isquestcompleted(questID)
			if cstring == nil then cstring = completed and COMPLETE or INCOMPLETE end
            s1, qt = GenerateLine(questID)
			s2 = qt == "R" and AddColor(L["repeatable"], DailyGlobalCheck_Options.Colors[9])                       -- repeatable
	                        or AddColor(cstring, completed and DailyGlobalCheck_Options.Colors[7] or DailyGlobalCheck_Options.Colors[8]) -- custom cstring or complete/incomplete
            print(s1.." "..s2)
		end
     end
  end
 end
end
---------------------------- push data -----------------------------------------
function DailyGlobalCheck:PushData(v, cfunc, plugin, refresh)
 setmeta(v, meta_qd_newindex)

 for qID, qData in pairs(v) do
  setmeta(qData, meta_qd_template) 
  
  local td = rawget(questsdata, qID)
  if (not td or td[PLUGIN] == plugin) and not rawget(questsdata_duplicates, tostring(qID)) then
   questsdata[qID] = qData
   questsdata[qID][COMPLETEDFUNC] = cfunc
   questsdata[qID][PLUGIN] = plugin
  elseif plugin then
   local old_data = questsdata[qID]
   questsdata[qID] = nil

   qID = tostring(qID)
   if not questsdata_duplicates[qID] then questsdata_duplicates[qID] = {} end

   if old_data then
    if old_data[PLUGIN] then
     questsdata_duplicates[qID][old_data[PLUGIN]] = old_data
	end
   end

   questsdata_duplicates[qID][plugin] = qData
   questsdata_duplicates[qID][plugin][COMPLETEDFUNC] = cfunc
   questsdata_duplicates[qID][plugin][PLUGIN] = plugin
  else
   --print("Daily Global Check - quest "..qID.." was not added because it already exists and cannot be a duplicate (no source plugin specified)")
  end
 end

 if refresh and SelectedList and SelectedList["Title"] == plugin then
  DailyGlobalCheck:Refresh()
  local _, list = lists:Find(plugin)
  maps:UpdateIcons("worldmap", list)
  C_Timer.After(0.5, function() maps:UpdateIcons("minimap", list) end)
 end
end

function DailyGlobalCheck:OpenList(v)
 local list = type(v) == "string" and select(2, lists:Find(v)) or v

 if not v or not lists:isList(list) then
  SelectedList = nil
  DailyGlobalCheck:Refresh()
  GenerateTomTomMenu()
  DailyGlobalCheck_PluginData["last_selected"] = nil
  if DGC_Mainframe then
   DGC_Mainframe:SelectListButton(nil)
  end
  return
 end

 SelectedList = list
 DailyGlobalCheck_PluginData["last_selected"] = list["Title"]
 GenerateTomTomMenu()
 DailyGlobalCheck:Refresh()
 if DGCMenuFrame then DGCMenuFrame:Hide() end
 if DGC_Mainframe then
  DGC_Mainframe:SelectListButton(SelectedList.Title)
 end
end
---------------------------------------------------------------------
local function GenerateMinimapMenu()
 if not DGCMMmenu then return end

  local function CreateListsMenu()
   local submenu = {}
    DailyGlobalCheck:foreachLists(function(k,v)
    table.insert(submenu, { text = v["Title"],
                            notCheckable = 1,
                            keepShownOnClick = true,
                            func = function()
                               DailyGlobalCheck:OpenList(v)
                              end
                              })
    end)
    return submenu
   end

  DGCMMmenu = {}
  table.insert(DGCMMmenu, {text = AddColor(L["availablelists"],"GOLD"),notCheckable = 1, hasArrow = true, menuList = CreateListsMenu()})
  
  if TomTom and DGCTTmenu then
   table.insert(DGCMMmenu, {text = AddColor("TomTom","GOLD"),notCheckable = 1, hasArrow = true, menuList = DGCTTmenu})
  end
  
  table.insert(DGCMMmenu, {text = "", isTitle = true, notCheckable = 1})

  table.insert(DGCMMmenu, {text = L["prevpage"], notCheckable = 1, func = function() 
   if pages:Move(SelectedList, -1) and DGC_Mainframe then
    DGC_Mainframe:SetAnchor("BOTTOMLEFT")
    DailyGlobalCheck:Refresh(SelectedList)
   end
  end})
  
  table.insert(DGCMMmenu, {text = L["nextpage"], notCheckable = 1, func = function() 
   if pages:Move(SelectedList, 1) and DGC_Mainframe then
    DGC_Mainframe:SetAnchor("BOTTOMLEFT")
    DailyGlobalCheck:Refresh(SelectedList)
   end
  end})
--  table.insert(DGCTTmenu, {text = "",notCheckable = 1, isTitle = true})
  table.insert(DGCMMmenu, {text = "", isTitle = true, notCheckable = 1})
  table.insert(DGCMMmenu, {text = CLOSE, notCheckable = 1})
  EasyMenu(DGCMMmenu, DGCMenuFrame, "cursor", -100 , 0, "MENU")
end

-- slash command
function DGCInit(arg)

 if arg == "print" then
  print(AddColor("Daily Global Check","LIGHT_BLUE"))
  local last_pID = DailyGlobalCheck.selectedpID
  DailyGlobalCheck.selectedpID = addonTable.pID
  ChatOutput()
  DailyGlobalCheck.selectedpID = last_pID
  print(AddColor("---","LIGHT_BLUE"))
  return
 end

 -- constructor moved to constr_mainframe.lua
 if not DGC_Mainframe then constructors:MainFrame() end

 local f = DGC_Mainframe
 
 local a1, _, a2, x, y = unpack(DailyGlobalCheck_Options["frame_position"] or {})
 if not a1 then
  f:SetPoint("CENTER")
 else
  f:SetPoint(a1, UIParent, a2, x, y)
 end
 
 if f:IsVisible() then
  f:Hide()
  if DGCMenuFrame then DGCMenuFrame:Hide() end
  collectgarbage()
 else
  modes:Set(C.MODE_CLASSIC)
  --modes:Set(C.MODE_EDIT) -- debug
  DailyGlobalCheck:Refresh()
  if SelectedList and SelectedList["Title"] then
   f:SelectListButton(SelectedList["Title"])
  end
  f:Show()
 end
end

local function updateMainframeButtons(enabled)
 for _,v in pairs(DGC_Mainframe.dynamic_buttons) do
  if enabled then
   v:Show()
  else
   v:Hide()
  end
 end
end

function DailyGlobalCheck:Refresh(list)
 local mode = DailyGlobalCheck.mode

 DailyGlobalCheck:RefreshTitle()
 DailyGlobalCheck:RefreshListButtons()

 if DGC_Mainframe then 
  DGC_Mainframe.closebtn:Show()
  DGC_Mainframe.dgcicon:SetTexture(T.dgcicon)
  if C.modes[mode].x and C.modes[mode].y then
   DGC_Mainframe:SetSize(C.modes[mode].x, C.modes[mode].y)
  end
 end

 if mode == C.MODE_CLASSIC then
  if not list or SelectedList == list then
   if DGC_Mainframe then
    DGC_Mainframe:UpdateArrows()
    DrawFrame()
	DGC_Mainframe.dgcicon:SetTexture(SelectedList and SelectedList["Icon"] or T.dgcicon)
   end
  end
  updateMainframeButtons(true)
 elseif mode == C.MODE_OPTIONS_QUEST or mode == C.MODE_OPTIONS_PLUGIN or mode == C.MODE_OPTIONS_MAP then -- options
  updateMainframeButtons(false)
  options:OnDraw(mode)
 elseif mode == C.MODE_LIST_VISIBILITY then -- list visibility
  updateMainframeButtons(false)
 elseif mode == C.MODE_EDIT then -- edit mode
  updateMainframeButtons(false)
  tools:update_edit_panel()
  DGC_Mainframe:UpdateArrows()
  DGC_Mainframe.Lbtn:Show()
  DGC_Mainframe.Rbtn:Show()
  DGC_Mainframe.dgcicon:SetTexture(SelectedList and SelectedList["Icon"] or T.dgcicon)
 elseif mode == C.MODE_EDIT_QUEST then
  DGC_Mainframe.closebtn:Hide()
 end
end

local ldbset = false
local eventframe = CreateFrame("FRAME","DGCEventFrame")
eventframe:RegisterEvent("PLAYER_LOGIN")

function eventframe.PLAYER_LOGIN(...)
  if DailyGlobalCheck_Options == nil then DailyGlobalCheck_Options = {} end
  
  local O = DailyGlobalCheck_Options

  -- NyxGUI init
  ng:Initialize(addonName, O, "main", C.default_theme)
  ng:SetThemeData(addonName, "main", C.default_theme) -- debug

  -- tmp
  if O["show_mapicons"] ~= nil then
   O["show_worldmapicons"] = O["show_mapicons"]
   O["show_mapicons"] = nil
  end  
  
  local function checkOpt(opt, subT)
   if subT then
    if O[subT][opt] == nil then O[subT][opt] = C.defaults[subT][opt] end 
   else
    if O[opt] == nil then O[opt] = C.defaults[opt] end 
   end
  end

  checkOpt("show_questtype")
  checkOpt("show_coordinates")
  checkOpt("show_zone")
  checkOpt("show_prefix")
  checkOpt("show_suffix")
  checkOpt("show_worldmapicons")
  checkOpt("show_minimapicons")
  checkOpt("show_questlogbtn")
  checkOpt("hide_completed")
  
  if O["chars_blacklist"]  == nil then O["chars_blacklist"]  = {} end
  if O["frame_position"]   == nil then O["frame_position"]   = {"CENTER"} end

  -- UI Options
  if not O.UI then O.UI = {} end
  checkOpt("worldmap_icon_interact", "UI")
  checkOpt("minimap_icon_interact", "UI")
  checkOpt("worldmap_icon_size", "UI")
  checkOpt("minimap_icon_size", "UI")
  --
  -- Color Options
  if not O.Colors then O.Colors = {} end
  for i = 1,9 do
   checkOpt(i, "Colors")
  end
  
  if not DGC_CustomLists.Lists then DGC_CustomLists.Lists = {} end
  if not DGC_CustomLists.QuestsData then DGC_CustomLists.QuestsData = {} end
  
  -- v1.2 --> v1.3 conversion
  if O["DGC_Version"] and O["DGC_Version"] < 1003 then
   O["last_selected"] = nil
   
   for k,v in pairs(DGC_CustomLists) do
    if lists:isList(v) then
	 for qID,QuestsData in pairs(v["Data"]) do
      DGC_CustomLists.QuestsData[qID] = QuestsData
	 end
	 v["Data"] = {}
     table.insert(DGC_CustomLists.Lists, v)
	 DGC_CustomLists[k] = nil
	end
   end
  end
  -- future conversions v1.3 --> x  here
  --
  
  -- metatables
  local meta_questsdata_duplicates = { __index = function(t, k)
                                                 k = tostring(k)
	                                             local questID, plugin = DailyGlobalCheck:NormalizeQuestID(k)
                                                 if plugin then
												 if not rawget(t, questID) then return end
												  return rawget(t, questID)[plugin]
												 elseif not rawget(t, k) then
												  return
												 else
												  return (SelectedList and rawget(t, k)[SelectedList.Title])
												 end
                                                end }

  local meta_qd_main_list   = { __newindex = meta_qd_template_newindex,
                                __index = DGC_CustomLists.QuestsData}

  local meta_qd_custom_list = { __newindex = meta_qd_template_newindex,
                                __index = questsdata_duplicates}
								 
  -- (main quests data > custom quests data > duplicates) cascade
  setmeta(DailyGlobalCheck.QuestsData, meta_qd_main_list)
  setmeta(DGC_CustomLists.QuestsData, meta_qd_custom_list)
  setmeta(questsdata_duplicates, meta_questsdata_duplicates)
  
  for k,v in pairs(DGC_CustomLists.QuestsData) do
   setmeta(v, meta_qd_template)
  end
  --

  DGCMenuFrame = CreateFrame("Frame", "DGCMenuFrame", UIParent, "UIDropDownMenuTemplate")

  -- plugins should've done loading
  C_Timer.After(3, function()
   maps:UpdateIcons("worldmap")
   maps:UpdateIcons("minimap")
   
   -- delete not existing lists from the character list settings saved var
   lists:UpdatePluginDataSVar()
  end)

  DailyGlobalCheck.initialized = true
  for k,v in pairs(DailyGlobalCheck.LoadQueue) do
   DailyGlobalCheck:LoadPlugin(k, v)
  end
  DailyGlobalCheck.LoadQueue = nil
  
  -- saved selected list
  local lastsel = DailyGlobalCheck_PluginData["last_selected"]
  if lastsel then
   DailyGlobalCheck:OpenList(lastsel)
  else
   local list = lists:FindFirst()
   DailyGlobalCheck:OpenList(list and list["Title"] or nil)
  end

  if O["minimap_icon"] == nil then -- show minimap icon
    O["minimap_icon"] = {
        hide = false,
        minimapPos = 220,
    }
  end
  O["DGC_Version"] = C.version

  if ldb and not ldbset then
        local DailyGlobalCheck_broker = ldb:NewDataObject("Daily Global Check", {
	        type = "data source",
	        icon = T.dgcicon,
	        label = "Daily Global Check",
	        OnClick = function(self,button)
			 if button == "LeftButton" then
               DGCInit()
			 elseif button == "RightButton" then
			  GenerateMinimapMenu()
			 end
	        end,
	        OnTooltipShow = function(tooltip)
			 if lists:isList(SelectedList) then
			  tooltip:ClearLines()
			  tooltip:AddLine(addontitle..AddColor(" ("..SelectedList["Title"]..")","LIGHT_BLUE"))
		      DrawTooltip(tooltip)
			 end
	    end,
        })
        if ldbicon then
            ldbicon:Register("DailyGlobalCheck_broker", DailyGlobalCheck_broker, O["minimap_icon"])
        end
   ldbset = true
 end
end

eventframe:SetScript("OnEvent", function(self, event, ...) eventframe[event](...) end)

-- "Add quest to Custom List" menubutton in the QuestMap frame
local function hookToggleDropDownMenu(level, value, dropDownFrame, anchorName, xOffset, yOffset, menuList, button, autoHideDelay)
 if dropDownFrame ~= QuestMapQuestOptionsDropDown or not DailyGlobalCheck_Options["show_questlogbtn"] then
  return
 end 

 local questLogIndex = GetQuestLogIndexByID(dropDownFrame.questID)
 local title, _, _, _, _, _, frequency, questID = GetQuestLogTitle(questLogIndex)
 local info = UIDropDownMenu_CreateInfo()

 info.isNotRadio = true
 info.notCheckable = true
 info.isTitle  = true

 -- blank space
 info.text = " "
 UIDropDownMenu_AddButton(info, level)
 -- addon title
 info.text = "|T"..T.dgcicon..":16|t "..addontitle
 UIDropDownMenu_AddButton(info, level)
 -- quest ID
 info.text = AddColor(L["b_questid"].." "..dropDownFrame.questID, "GOLD")
 UIDropDownMenu_AddButton(info, level)
 -- add buttons
 info.isTitle  = false
 info.disabled = false
 info.text = L["add_quest"]
 info.func = function()

  if DailyGlobalCheck:getQData(questID) then
   print(L["bf_err2"])
   return
  end

  local data = {}
  local map = GetQuestWorldMapAreaID(questID)
  data[ZONE] = map and GetMapNameByID(map) or ""
  data[NAME] = title
  data[PREF] = ""
  data[SUFF] = ""
  data[QUESTTYPE] = frequency == 1 and "Q" or frequency == LE_QUEST_FREQUENCY_WEEKLY and "W" or nil
  DGC_CustomLists.QuestsData[questID] = data
  print(string.format(L["questlog_quest_added"], AddColor(title, "NAME_COLOR")))
 end
 UIDropDownMenu_AddButton(info, level)
end

hooksecurefunc("ToggleDropDownMenu", hookToggleDropDownMenu)

-- slash command
SLASH_DAILYGLOBALCHECK1 = "/dgcheck"
SLASH_DAILYGLOBALCHECK2 = "/dgc"
SlashCmdList["DAILYGLOBALCHECK"] = DGCInit