-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

--[[ v1.3 lists functions ]]--

local addonName, addonTable = ...
local L = addonTable.L

local lists = DailyGlobalCheck.ListFunctions
local pages = DailyGlobalCheck.PagesFunctions
local maps  = DailyGlobalCheck.MapFunctions
local dgcicon = "Interface\\icons\\INV_Sigil_Freya"

function lists:isList(v)
 if not v then return end
 
 return (v["Title"] and v["Icon"] and v["Order"])
end

function lists:isPlugin(title)
 local _,_,container = lists:Find(title, false)
 return container == DailyGlobalCheck.Lists
end

function lists:GetSvar(v)
 if not lists:isList(v) then return end

 if not DailyGlobalCheck_PluginData[v["Title"]] then
  DailyGlobalCheck_PluginData[v["Title"]] = {}
 end
 
 return DailyGlobalCheck_PluginData[v["Title"]]
end

------- active lists -------
function lists:isActive(v)
 if not lists:isList(v) or not DailyGlobalCheck_PluginData[v["Title"]] then 
  return
 else
  return DailyGlobalCheck_PluginData[v["Title"]].active
 end
end

function lists:setActive(v, bool)
 if not lists:isList(v) then return end

 if not DailyGlobalCheck_PluginData[v["Title"]] then
  DailyGlobalCheck_PluginData[v["Title"]] = {}
 end

 DailyGlobalCheck_PluginData[v["Title"]].active = bool
 maps:UpdateIcons("worldmap", v)
 maps:UpdateIcons("minimap", v)
end
----------------------------

------- show plugin worldmap and minimap icons -------
function lists:getShowMapIcons(v, t)
 if not lists:isList(v) then return end
 
 local svar = lists:GetSvar(v)
 return svar["show_mapicons_"..t] == nil or svar["show_mapicons_"..t]
end

function lists:setShowMapIcons(v, t, bool)
 if not lists:isList(v) then return end

 local svar = lists:GetSvar(v)

 svar["show_mapicons_"..t] = bool
 maps:UpdateIcons(t, v)
end
------------------------------------------------------

function lists:Count()
 return #(DailyGlobalCheck.Lists) + #(DGC_CustomLists.Lists)
end

function lists:FindFirst(active)

 if active == false then
  return DailyGlobalCheck.Lists[1] or DGC_CustomLists.Lists[1]
 end

 local i = 1
 local container = DailyGlobalCheck.Lists

 while i <= #container do
  if lists:isActive(container[i]) then
   return container[i]
  end
  i = i + 1
 end
 
 i = 1
 container = DGC_CustomLists.Lists
 
 while i <= #container do
  if lists:isActive(container[i]) then
   return container[i]
  end
  i = i + 1
 end
end

function lists:Find(title, active)

 for k,v in pairs(DailyGlobalCheck.Lists) do
  if v.Title == title and (active == false or lists:isActive(v)) then
   return k, v, DailyGlobalCheck.Lists
  end
 end

 if not DGC_CustomLists.Lists then return end
 
 for k,v in pairs(DGC_CustomLists.Lists) do
  if v.Title == title and (active == false or lists:isActive(v)) then
   return k, v, DGC_CustomLists.Lists
  end
 end
end

function lists:CheckVersion(v)
 return v.Version and v.Version <= DailyGlobalCheck:GetVersion()
end

function lists:New()
 local title = L["newlist"]
 local counter = 1
 
 while lists:Find(title, false) do
  title = L["newlist"].." ("..counter..")"
  counter = counter + 1
 end

 local result = {}
 result.Title   = title
 result.Icon    = dgcicon
 result.Version = DailyGlobalCheck:GetVersion()
 result.Order   = {}
 pages:New(result)
 result.MultiCharsEnabled = true
 return result
end

function lists:Rename(list, newtitle)
 
 local oldtitle = list.Title
 list.Title = newtitle

 if not DailyGlobalCheck_PluginData[oldtitle] then return end
 
 local old_list = DailyGlobalCheck_PluginData[oldtitle]
 DailyGlobalCheck_PluginData[oldtitle] = nil
 DailyGlobalCheck_PluginData[newtitle] = old_list
 
 if DailyGlobalCheck_PluginData.last_selected and DailyGlobalCheck_PluginData.last_selected == oldtitle then
  DailyGlobalCheck_PluginData.last_selected = newtitle
 end
end

function lists:Remove(title)
 local i, _, lists = lists:Find(title, false)
 
 if not i then return end
 
 table.remove(lists, i)
end

function lists:ContainsQuest(v, questID)
 if not lists:isList(v) then return end

 for pp,pages in pairs(v.Order) do
  for pi,page in pairs(pages) do
   for i,qID in pairs(page) do
    if qID == questID then
	 return pp, pi, i
	end
   end
  end
 end
end

function lists:UpdatePluginDataSVar()
 for k,v in pairs(DailyGlobalCheck_PluginData) do
  if k ~= "last_selected" and not lists:Find(k, false) then
   DailyGlobalCheck_PluginData[k] = nil
  end
 end
end